@@ -17,7 +17,20 @@
  * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <xfs.h>
+#include "libxfs_priv.h"
+#include "xfs_fs.h"
+#include "xfs_shared.h"
+#include "xfs_format.h"
+#include "xfs_log_format.h"
+#include "xfs_trans_resv.h"
+#include "xfs_mount.h"
+#include "xfs_inode_buf.h"
+#include "xfs_inode_fork.h"
+#include "xfs_inode.h"
+#include "xfs_trans.h"
+#include "xfs_sb.h"
+
+static void xfs_trans_free_items(struct xfs_trans *tp);
 
 /*
  * Simple transaction interface
@@ -90,22 +103,26 @@ libxfs_trans_roll(
 	struct xfs_trans	**tpp,
 	struct xfs_inode	*dp)
 {
+	struct xfs_mount	*mp;
 	struct xfs_trans	*trans;
 	struct xfs_trans_res	tres;
+	unsigned int		old_blk_res;
 	int			error;
 
 	/*
 	 * Ensure that the inode is always logged.
 	 */
 	trans = *tpp;
-	xfs_trans_log_inode(trans, dp, XFS_ILOG_CORE);
+	if (dp)
+		xfs_trans_log_inode(trans, dp, XFS_ILOG_CORE);
 
 	/*
 	 * Copy the critical parameters from one trans to the next.
 	 */
+	mp = trans->t_mountp;
 	tres.tr_logres = trans->t_log_res;
 	tres.tr_logcount = trans->t_log_count;
-	*tpp = xfs_trans_dup(trans);
+	old_blk_res = trans->t_blk_res;
 
 	/*
 	 * Commit the current transaction.
@@ -114,11 +131,10 @@ libxfs_trans_roll(
 	 * is in progress. The caller takes the responsibility to cancel
 	 * the duplicate transaction that gets returned.
 	 */
-	error = xfs_trans_commit(trans, 0);
+	error = xfs_trans_commit(trans);
 	if (error)
-		return (error);
+		return error;
 
-	trans = *tpp;
 
 	/*
 	 * Reserve space in the log for th next transaction.
@@ -129,59 +145,33 @@ libxfs_trans_roll(
 	 * the prior and the next transactions.
 	 */
 	tres.tr_logflags = XFS_TRANS_PERM_LOG_RES;
-	error = xfs_trans_reserve(trans, &tres, 0, 0);
+	error = libxfs_trans_alloc(mp, &tres, 0, 0, 0, tpp);
+	trans = *tpp;
+	trans->t_blk_res = old_blk_res;
+
 	/*
 	 *  Ensure that the inode is in the new transaction and locked.
 	 */
 	if (error)
 		return error;
 
-	xfs_trans_ijoin(trans, dp, 0);
+	if (dp)
+		xfs_trans_ijoin(trans, dp, 0);
 	return 0;
 }
 
-xfs_trans_t *
-libxfs_trans_alloc(
-	xfs_mount_t	*mp,
-	int		type)
-{
-	xfs_trans_t	*ptr;
-
-	if ((ptr = calloc(sizeof(xfs_trans_t), 1)) == NULL) {
-		fprintf(stderr, _("%s: xact calloc failed (%d bytes): %s\n"),
-			progname, (int)sizeof(xfs_trans_t), strerror(errno));
-		exit(1);
-	}
-	ptr->t_mountp = mp;
-	ptr->t_type = type;
-	INIT_LIST_HEAD(&ptr->t_items);
-#ifdef XACT_DEBUG
-	fprintf(stderr, "allocated new transaction %p\n", ptr);
-#endif
-	return ptr;
-}
-
-xfs_trans_t *
-libxfs_trans_dup(
-	xfs_trans_t	*tp)
-{
-	xfs_trans_t	*ptr;
-
-	ptr = libxfs_trans_alloc(tp->t_mountp, tp->t_type);
-#ifdef XACT_DEBUG
-	fprintf(stderr, "duplicated transaction %p (new=%p)\n", tp, ptr);
-#endif
-	return ptr;
-}
-
 int
-libxfs_trans_reserve(
-	struct xfs_trans	*tp,
+libxfs_trans_alloc(
+	struct xfs_mount	*mp,
 	struct xfs_trans_res	*resp,
-	uint			blocks,
-	uint			rtextents)
+	unsigned int		blocks,
+	unsigned int		rtextents,
+	unsigned int		flags,
+	struct xfs_trans	**tpp)
+
 {
-	xfs_sb_t	*mpsb = &tp->t_mountp->m_sb;
+	struct xfs_sb	*sb = &mp->m_sb;
+	struct xfs_trans *ptr;
 
 	/*
 	 * Attempt to reserve the needed disk blocks by decrementing
@@ -189,23 +179,56 @@ libxfs_trans_reserve(
 	 * fail if the count would go below zero.
 	 */
 	if (blocks > 0) {
-		if (mpsb->sb_fdblocks < blocks)
-			return ENOSPC;
+		if (sb->sb_fdblocks < blocks)
+			return -ENOSPC;
 	}
-	/* user space, don't need log/RT stuff (preserve the API though) */
+
+	if ((ptr = calloc(sizeof(xfs_trans_t), 1)) == NULL) {
+		fprintf(stderr, _("%s: xact calloc failed (%d bytes): %s\n"),
+			progname, (int)sizeof(xfs_trans_t), strerror(errno));
+		exit(1);
+	}
+	ptr->t_mountp = mp;
+	ptr->t_blk_res = blocks;
+	INIT_LIST_HEAD(&ptr->t_items);
+#ifdef XACT_DEBUG
+	fprintf(stderr, "allocated new transaction %p\n", ptr);
+#endif
+	*tpp = ptr;
 	return 0;
 }
 
+/*
+ * Create an empty transaction with no reservation.  This is a defensive
+ * mechanism for routines that query metadata without actually modifying
+ * them -- if the metadata being queried is somehow cross-linked (think a
+ * btree block pointer that points higher in the tree), we risk deadlock.
+ * However, blocks grabbed as part of a transaction can be re-grabbed.
+ * The verifiers will notice the corrupt block and the operation will fail
+ * back to userspace without deadlocking.
+ *
+ * Note the zero-length reservation; this transaction MUST be cancelled
+ * without any dirty data.
+ */
+int
+libxfs_trans_alloc_empty(
+	struct xfs_mount		*mp,
+	struct xfs_trans		**tpp)
+{
+	struct xfs_trans_res		resv = {0};
+
+	return xfs_trans_alloc(mp, &resv, 0, 0, XFS_TRANS_NO_WRITECOUNT, tpp);
+}
+
 void
 libxfs_trans_cancel(
-	xfs_trans_t	*tp,
-	int		flags)
+	xfs_trans_t	*tp)
 {
 #ifdef XACT_DEBUG
 	xfs_trans_t	*otp = tp;
 #endif
 	if (tp != NULL) {
-		xfs_trans_free_items(tp, flags);
+		xfs_trans_free_items(tp);
 		free(tp);
 		tp = NULL;
 	}
@@ -228,9 +251,9 @@ libxfs_trans_iget(
 	xfs_inode_log_item_t	*iip;
 
 	if (tp == NULL)
-		return libxfs_iget(mp, tp, ino, lock_flags, ipp, 0);
+		return libxfs_iget(mp, tp, ino, lock_flags, ipp);
 
-	error = libxfs_iget(mp, tp, ino, lock_flags, &ip, 0);
+	error = libxfs_iget(mp, tp, ino, lock_flags, &ip);
 	if (error)
 		return error;
 	ASSERT(ip != NULL);
@@ -560,8 +583,7 @@ libxfs_trans_read_buf_map(
 	if (tp == NULL) {
 		bp = libxfs_readbuf_map(btp, map, nmaps, flags, ops);
 		if (!bp) {
-			return (flags & XBF_TRYLOCK) ?
-				EAGAIN : XFS_ERROR(ENOMEM);
+			return (flags & XBF_TRYLOCK) ?  -EAGAIN : -ENOMEM;
 		}
 		if (bp->b_error)
 			goto out_relse;
@@ -579,8 +601,7 @@ libxfs_trans_read_buf_map(
 
 	bp = libxfs_readbuf_map(btp, map, nmaps, flags, ops);
 	if (!bp) {
-		return (flags & XBF_TRYLOCK) ?
-			EAGAIN : XFS_ERROR(ENOMEM);
+		return (flags & XBF_TRYLOCK) ?  -EAGAIN : -ENOMEM;
 	}
 	if (bp->b_error)
 		goto out_relse;
@@ -734,7 +755,7 @@ trans_committed(
         list_for_each_entry_safe(lidp, next, &tp->t_items, lid_trans) {
 		struct xfs_log_item *lip = lidp->lid_item;
 
-                xfs_trans_del_item(lip);
+		xfs_trans_del_item(lip);
 
 		if (lip->li_type == XFS_LI_BUF)
 			buf_item_done((xfs_buf_log_item_t *)lip);
@@ -780,10 +801,9 @@ inode_item_unlock(
  * Unlock all of the items of a transaction and free all the descriptors
  * of that transaction.
  */
-void
+static void
 xfs_trans_free_items(
-	struct xfs_trans	*tp,
-	int			flags)
+	struct xfs_trans	*tp)
 {
 	struct xfs_log_item_desc *lidp, *next;
 
@@ -808,8 +828,7 @@ xfs_trans_free_items(
  */
 int
 libxfs_trans_commit(
-	xfs_trans_t	*tp,
-	uint		flags)
+	xfs_trans_t	*tp)
 {
 	xfs_sb_t	*sbp;
 
@@ -820,7 +839,7 @@ libxfs_trans_commit(
 #ifdef XACT_DEBUG
 		fprintf(stderr, "committed clean transaction %p\n", tp);
 #endif
-		xfs_trans_free_items(tp, flags);
+		xfs_trans_free_items(tp);
 		free(tp);
 		tp = NULL;
 		return 0;
@@ -836,7 +855,7 @@ libxfs_trans_commit(
 			sbp->sb_fdblocks += tp->t_fdblocks_delta;
 		if (tp->t_frextents_delta)
 			sbp->sb_frextents += tp->t_frextents_delta;
-		xfs_mod_sb(tp, XFS_SB_ALL_BITS);
+		xfs_log_sb(tp);
 	}
 
 #ifdef XACT_DEBUG
